/*
 * Copyright (c) 2012 Adam Hraska
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 * - The name of the author may not be used to endorse or promote products
 *   derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef KERN_COMPILER_BARRIER_H_
#define KERN_COMPILER_BARRIER_H_

#include <stdatomic.h>
#include <stddef.h>

static inline void compiler_barrier(void)
{
	atomic_signal_fence(memory_order_seq_cst);
}

static inline void memory_barrier(void)
{
	atomic_thread_fence(memory_order_seq_cst);
}

static inline void read_barrier(void)
{
	atomic_thread_fence(memory_order_acquire);
}

static inline void write_barrier(void)
{
	atomic_thread_fence(memory_order_release);
}

#define CS_ENTER_BARRIER() atomic_thread_fence(memory_order_acquire)
#define CS_LEAVE_BARRIER() atomic_thread_fence(memory_order_release)

/** Forces the compiler to access (ie load/store) the variable only once. */
#define ACCESS_ONCE(var) (*((volatile typeof(var)*)&(var)))

extern void smc_coherence(void *, size_t);

#endif /* KERN_COMPILER_BARRIER_H_ */
